# -*- coding: utf-8 -*-

"""
twython.exceptions
~~~~~~~~~~~~~~~~~~

This module contains Twython specific Exception classes.
"""

from .endpoints import TWITTER_HTTP_STATUS_CODE


class TwythonError(Exception):
    """Generic error class, catch-all for most Twython issues.
    Special cases are handled by TwythonAuthError & TwythonRateLimitError.

    from twython import TwythonError, TwythonRateLimitError, TwythonAuthError

    """
    def __init__(self, msg, error_code=None, retry_after=None):
        self.error_code = error_code

        if error_code is not None and error_code in TWITTER_HTTP_STATUS_CODE:
            msg = 'Twitter API returned a %s (%s), %s' % \
                  (error_code,
                   TWITTER_HTTP_STATUS_CODE[error_code][0],
                   msg)

        super(TwythonError, self).__init__(msg)

    @property
    def msg(self):  # pragma: no cover
        return self.args[0]


class TwythonAuthError(TwythonError):
    """Raised when you try to access a protected resource and it fails due to
    some issue with your authentication.

    """
    pass


class TwythonRateLimitError(TwythonError):  # pragma: no cover
    """Raised when you've hit a rate limit.

    The amount of seconds to retry your request in will be appended
    to the message.

    """
    def __init__(self, msg, error_code, retry_after=None):
        if isinstance(retry_after, int):
            msg = '%s (Retry after %d seconds)' % (msg, retry_after)
        TwythonError.__init__(self, msg, error_code=error_code)

        self.retry_after = retry_after


class TwythonStreamError(TwythonError):
    """Raised when an invalid response from the Stream API is received"""
    pass



# -*- coding: utf-8 -*-

"""
twython.compat
~~~~~~~~~~~~~~

This module contains imports and declarations for seamless Python 2 and
Python 3 compatibility.
"""

import sys

_ver = sys.version_info

#: Python 2.x?
is_py2 = (_ver[0] == 2)

#: Python 3.x?
is_py3 = (_ver[0] == 3)

try:
    import simplejson as json
except ImportError:
    import json

if is_py2:
    from urllib import urlencode, quote_plus
    from urlparse import parse_qsl, urlsplit

    str = unicode
    basestring = basestring
    numeric_types = (int, long, float)


elif is_py3:
    from urllib.parse import urlencode, quote_plus, parse_qsl, urlsplit

    str = str
    basestring = (str, bytes)
    numeric_types = (int, float)



# -*- coding: utf-8 -*-

"""
twython.helpers
~~~~~~~~~~~~~~~

This module contains functions that are repeatedly used throughout
the Twython library.
"""

from .compat import basestring, numeric_types


def _transparent_params(_params):
    params = {}
    files = {}
    for k, v in _params.items():
        if hasattr(v, 'read') and callable(v.read):
            files[k] = v  # pragma: no cover
        elif isinstance(v, bool):
            if v:
                params[k] = 'true'
            else:
                params[k] = 'false'
        elif isinstance(v, basestring) or isinstance(v, numeric_types):
            params[k] = v
        elif isinstance(v, list):
            try:
                params[k] = ','.join(v)
            except TypeError:
                params[k] = ','.join(map(str, v))
        else:
            continue  # pragma: no cover
    return params, files
